package fr.natan.akkastreamfileprocessingapi.service

import akka.actor.ActorSystem
import akka.stream.alpakka.csv.scaladsl.{CsvParsing, CsvToMap}
import akka.stream.scaladsl.{Compression, FileIO, Flow, Sink, Source}
import akka.{Done, NotUsed}
import com.typesafe.scalalogging.slf4j.Logger
import fr.natan.akkastreamfileprocessingapi.businessexceptions.FileNotFoundException
import fr.natan.akkastreamfileprocessingapi.models.Models.{Person, TvSeries}
import fr.natan.akkastreamfileprocessingapi.valitator.Validators.fileExists
import fr.natan.akkastreamfileprocessingapi.models.ModelsBuilder.{buildPersonModel, buildTvSerieModel}

import java.io.File
import java.nio.file.Paths
import scala.concurrent.Future

object AkkaStreamComponents {

  implicit val actor: ActorSystem = ActorSystem("AkkaStreamActor")

  //flows building

  //flow1
  def buildTvSerieFlow(): Flow[Map[String, String], TvSeries, NotUsed] = {

    val tvFlow: Flow[Map[String, String], TvSeries, NotUsed] =
      Flow[Map[String, String]]
        .map(row => {
          buildTvSerieModel(row)
        })
    tvFlow
  }

  //flow2
  def buildPersonFlow(): Flow[Map[String, String], Person, NotUsed] = {
    val personFlow: Flow[Map[String, String], Person, NotUsed] =
      Flow[Map[String, String]]
        .map((rowMap: Map[String, String]) => {
          buildPersonModel(rowMap)
        })

    personFlow
  }

  //flow3
  def filterByMoviePrimaryTitleFlow(moviePrimaryTitle: String): Flow[Map[String, String], TvSeries, NotUsed] = {
    val filterFlow: Flow[Map[String, String], TvSeries, NotUsed] = Flow[Map[String, String]]
        .filter((rows: Map[String, String]) => {
          rows.getOrElse("primaryTitle","")==moviePrimaryTitle
        })
        .map(rowMap => {
          buildTvSerieModel(map = rowMap)
        })

    filterFlow
  }

  //flow4
  def filterByPersonIdFlow(nconst: String): Flow[Map[String, String], Person, NotUsed]={
    val personFilter: Flow[Map[String, String], Person, NotUsed]=
      Flow[Map[String, String]]
        .filter((rowMap:Map[String, String])=>{
          rowMap.getOrElse("nconst","")==nconst
        })
        .map(rowMap=>{
          buildPersonModel(map = rowMap)
        })

    personFilter
  }

  def filterByPersonNameFlow(personName: String): Flow[Map[String, String], Person, NotUsed] ={
    val personFilter: Flow[Map[String, String], Person, NotUsed] =
      Flow[Map[String, String]]
        .filter((rowMap: Map[String, String]) =>{
          rowMap.getOrElse("primaryName","")==personName
        })
        .map((rowMap: Map[String, String])=>{
          buildPersonModel(map = rowMap)
        })

    personFilter
  }

  //source building
  def buildSource(inputFile: File): Source[Map[String, String], NotUsed] = {

    var datasource: Source[Map[String, String], NotUsed] = null

    if (!fileExists(inputFile.getPath)) {
      return null
    }
    datasource = Source
      .single(inputFile)
      .flatMapConcat((filename: File) => {
        FileIO.fromPath(
          Paths.get(filename.getPath)
        )
      }
      )
      .via(Compression.gunzip())
      .via(CsvParsing.lineScanner(CsvParsing.Tab, CsvParsing.DoubleQuote))
      .via(CsvToMap.toMapAsStrings())

    datasource
  }

  def buildAndValidateSource(inputFile: File): Source[Map[String, String], NotUsed] = {

    val source: Source[Map[String, String], NotUsed] = buildSource(inputFile = inputFile)
    if (source == null) {
      throw new FileNotFoundException(filename = inputFile.getPath)
    }
    source
  }

  //sinks building

  //sink1
  def buildTvSeriesSink(logger: Logger): Sink[TvSeries, Future[Done]] = {
    val tvSeriesSink : Sink[TvSeries, Future[Done]]= Sink.foreach[TvSeries](
      (movie: TvSeries) => {
        logger.info(s"${movie.toString}")
      }
    )
    tvSeriesSink
  }

  //sink2
  def buildAllPersonsSink(logger: Logger): Sink[Person,Future[Seq[Person]]] = {
    val listPersonsSink: Sink[Person, Future[Seq[Person]]]=
      Sink.seq
    listPersonsSink
  }

  //sink3
  def buildPersonSink(logger: Logger): Sink[Person, Future[Done]] = {
    Sink.foreach[Person](
      (person: Person) => logger.info(s"${person.toString}")
    )
  }
}
